@@ -1,16 +1,18 @@
 package Reddit::Client;
 
-our $VERSION = '0.04_02';
+our $VERSION = '0.8_1';
 $VERSION = eval $VERSION;
 
+use strict;
+use warnings;
 use Carp;
+
 use Data::Dumper   qw/Dumper/;
 use JSON           qw//;
-use LWP::UserAgent qw//;
-use HTTP::Request  qw//;
 use File::Spec     qw//;
 use Digest::MD5    qw/md5_hex/;
 use POSIX          qw/strftime/;
+use File::Path::Expand qw//;
 
 require Reddit::Client::Account;
 require Reddit::Client::Comment;
@@ -91,11 +93,11 @@ $API[API_DEL        ] = ['POST', '/api/del'       ];
 
 sub DEBUG {
     if ($DEBUG) {
-	    my ($format, @args) = @_;
-	    my $ts  = strftime "%a %b %e %H:%M:%S %Y", localtime;
-	    my $msg = sprintf $format, @args;
-	    chomp $msg;
-	    printf STDERR "[%s] [ %s ]\n", $ts, $msg;
+        my ($format, @args) = @_;
+        my $ts  = strftime "%Y-%m-%d %H:%M:%S", localtime;
+        my $msg = sprintf $format, @args;
+        chomp $msg;
+        printf STDERR "[%s] [ %s ]\n", $ts, $msg;
     }
 }
 
@@ -109,7 +111,7 @@ sub subreddit {
         if ($subject eq '') { # front page
             return '';
         } else {              # subreddit
-	        return $subject;
+            return $subject;
         }
     } else { # fail
         return;
@@ -124,12 +126,19 @@ use fields (
     'modhash',      # store session modhash
     'cookie',       # store user cookie
     'session_file', # path to session file
+    'user_agent',   # user agent string
 );
 
 sub new {
     my ($class, %param) = @_;
     my $self = fields::new($class);
 
+    if (not exists $param{user_agent}) {
+        carp "Reddit::Client->new: user_agent required in future version.";
+        $param{user_agent} = $UA;
+    }
+    $self->{user_agent} = $param{user_agent};
+
     if ($param{session_file}) {
         $self->{session_file} = $param{session_file};
         $self->load_session;
@@ -148,7 +157,7 @@ sub request {
     $path =~ s/^\/+//;
 
     my $request = Reddit::Client::Request->new(
-        user_agent => $UA,
+        user_agent => $self->{user_agent},
         url        => sprintf('%s/%s', $BASE_URL, $path),
         method     => $method,
         query      => $query,
@@ -166,9 +175,9 @@ sub json_request {
 
     if ($method eq 'POST') {
         $post_data ||= {};
-	    $post_data->{api_type} = 'json';
+        $post_data->{api_type} = 'json';
     } else {
-	    $path .= '.json';
+        $path .= '.json';
     }
 
     my $response = $self->request($method, $path, $query, $post_data);
@@ -248,7 +257,9 @@ sub save_session {
 
     # Prepare session and file path
     my $session   = { modhash => $self->{modhash}, cookie => $self->{cookie} };
-    my $file_path = defined $file ? $file : $self->{session_file};
+    my $file_path = File::Path::Expand::expand_filename(
+        defined $file ? $file : $self->{session_file}
+    );
 
     DEBUG('Save session to %s', $file_path);
 
@@ -266,7 +277,9 @@ sub save_session {
 sub load_session {
     my ($self, $file) = @_;
     $self->{session_file} || $file || croak 'Expected $file';
-    my $file_path = defined $file ? $file : $self->{session_file};
+    my $file_path = File::Path::Expand::expand_filename(
+        defined $file ? $file : $self->{session_file}
+    );
 
     DEBUG('Load session from %s', $file_path);
 
@@ -329,9 +342,9 @@ sub list_subreddits {
     defined $type || croak 'Expected $type"';
 
     $self->require_login
-	    if $type eq SUBREDDITS_MOD
-	    || $type eq SUBREDDITS_MINE
-	    || $type eq SUBREDDITS_CONTRIB;
+        if $type eq SUBREDDITS_MOD
+        || $type eq SUBREDDITS_MINE
+        || $type eq SUBREDDITS_CONTRIB;
 
     my $result = $self->api_json_request(api => API_SUBREDDITS, args => [$type]);
     return {
@@ -382,15 +395,16 @@ sub fetch_links {
 
     my $query  = {};
     if ($before || $after || $limit) {
-	    $query->{limit}  = $limit  if defined $limit;
-	    $query->{before} = $before if defined $before;
-	    $query->{after}  = $after  if defined $after;
+        $query->{limit}  = $limit  if defined $limit;
+        $query->{before} = $before if defined $before;
+        $query->{after}  = $after  if defined $after;
     }
 
     $subreddit = subreddit($subreddit);
-    $args = [$view];
-    push @$args, $subreddit if $subreddit;
-    
+
+    my $args = [$view];
+    unshift @$args, $subreddit if $subreddit;
+
     my $result = $self->api_json_request(
         api      => ($subreddit ? API_LINKS_FRONT : API_LINKS_OTHER),
         args     => $args,
@@ -413,9 +427,9 @@ sub delete_item {
     my ($self, %param) = @_;
     my $name = $param{name} || croak 'Expected "name"';
     $self->require_login;
-    
+
     DEBUG('Delete post/comment %s', $name);
-    
+
     my $result = $self->api_json_request(api => API_DEL, data => { id => $name });
     return 1;
 }
@@ -557,16 +571,15 @@ __END__
 
 Reddit::Client - A perl wrapper for Reddit
 
-=head1 VERSION
-
-Version 0.01
-
 =head1 SYNOPSIS
 
     use Reddit::Client;
 
     my $session_file = '~/.reddit';
-    my $reddit       = Reddit::Client->new(session_file => $session_file);
+    my $reddit       = Reddit::Client->new(
+        session_file => $session_file,
+        user_agent   => 'MyApp/1.0',
+    );
 
     unless ($reddit->is_logged_in) {
         $reddit->login('someone', 'secret');
@@ -595,24 +608,24 @@ For more information about the Reddit API, see L<https://github.com/reddit/reddi
 
 =head1 CONSTANTS
 
-	VIEW_HOT            "Hot" links feed
-	VIEW_NEW            "New" links feed
-	VIEW_CONTROVERSIAL  "Controversial" links feed
-	VIEW_TOP            "Top" links feed
-	
-	VIEW_DEFAULT        Default feed if not specified (VIEW_HOT)
-	DEFAULT_LIMIT       The default number of links to be retried (25)
-	
-	VOTE_UP             Up vote
-	VOTE_DOWN           Down vote
-	VOTE_NONE           "Un" vote
+    VIEW_HOT            "Hot" links feed
+    VIEW_NEW            "New" links feed
+    VIEW_CONTROVERSIAL  "Controversial" links feed
+    VIEW_TOP            "Top" links feed
 
-	SUBREDDITS_HOME     List reddits on the homepage
-	SUBREDDITS_POPULAR  List popular reddits
-	SUBREDDITS_NEW      List new reddits
-	SUBREDDITS_MINE     List reddits for which the logged in user is subscribed
-	SUBREDDITS_CONTRIB  List reddits for which the logged in user is a contributor
-	SUBREDDITS_MOD      List reddits for which the logged in user is a moderator
+    VIEW_DEFAULT        Default feed if not specified (VIEW_HOT)
+    DEFAULT_LIMIT       The default number of links to be retried (25)
+
+    VOTE_UP             Up vote
+    VOTE_DOWN           Down vote
+    VOTE_NONE           "Un" vote
+
+    SUBREDDITS_HOME     List reddits on the homepage
+    SUBREDDITS_POPULAR  List popular reddits
+    SUBREDDITS_NEW      List new reddits
+    SUBREDDITS_MINE     List reddits for which the logged in user is subscribed
+    SUBREDDITS_CONTRIB  List reddits for which the logged in user is a contributor
+    SUBREDDITS_MOD      List reddits for which the logged in user is a moderator
 
 =head1 GLOBALS
 
@@ -621,6 +634,7 @@ For more information about the Reddit API, see L<https://github.com/reddit/reddi
 =item $UA
 
 This is the user agent string, and defaults to C<Reddit::Client/$VERSION>.
+NOTE: This is now deprecated in favor of the user_agent argument to new().
 
 
 =item $DEBUG
@@ -634,12 +648,13 @@ When set to true, outputs a small amount of debugging information.
 
 =over
 
-=item new(session_file => ...)
-
-Begins a new or loads an existing reddit session. If C<session_file> is
-provided, it will be read and parsed as JSON. If session data is found, it
-is restored. Otherwise, a new session is started.
+=item new(user_agent => ..., session_file => ...)
 
+Begins a new or loads an existing reddit session. The C<user_agent> argument
+will be required in a future release. Omitting it will generate a warning.
+If C<session_file> is provided, it will be read and parsed as JSON. If
+session data is found, it is restored. Otherwise, a new session is started.
+Session data does not restore the user_agent string of the original session.
 
 =item is_logged_in
 
